@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 /*
   Copyright 2005 Roberto Raggi <roberto@kdevelop.org>
@@ -52,6 +30,8 @@
 #include <cplusplus/Literals.h>
 #include <cplusplus/cppassert.h>
 
+#include <utils/executeondestruction.h>
+#include <utils/filepath.h>
 #include <utils/scopedswap.h>
 
 #include <QDebug>
@@ -77,6 +57,7 @@ using namespace Utils;
 namespace {
 enum {
     MAX_FUNCTION_LIKE_ARGUMENTS_COUNT = 100,
+    MAX_INCLUDE_DEPTH = 200,
     MAX_TOKEN_EXPANSION_COUNT = 5000,
     MAX_TOKEN_BUFFER_DEPTH = 16000 // for when macros are using some kind of right-folding, this is the list of "delayed" buffers waiting to be expanded after the current one.
 };
@@ -161,6 +142,7 @@ namespace Internal {
 struct TokenBuffer
 {
     std::deque<PPToken> tokens;
+    std::vector<QByteArray> blockedMacroNames;
     const Macro *macro;
     TokenBuffer *next;
 
@@ -172,10 +154,14 @@ struct TokenBuffer
         if (!macro)
             return false;
 
-        for (const TokenBuffer *it = this; it; it = it->next)
-            if (it->macro)
-                if (it->macro == macro || (it->macro->name() == macro->name()))
-                    return true;
+        for (const TokenBuffer *it = this; it; it = it->next) {
+            if (it->macro && (it->macro == macro || it->macro->name() == macro->name()))
+                return true;
+        }
+        for (const QByteArray &blockedMacroName : blockedMacroNames) {
+            if (macro->name() == blockedMacroName)
+                return true;
+        }
         return false;
     }
 };
@@ -217,6 +203,24 @@ struct Value
     inline bool is_zero () const
     { return l == 0; }
 
+    template<typename T> static int cmpImpl(T v1, T v2)
+    {
+        if (v1 < v2)
+            return -1;
+        if (v1 > v2)
+            return 1;
+        return 0;
+    }
+    Value cmp(const Value &other) const
+    {
+        Value v = *this;
+        if (v.is_ulong() || other.is_ulong())
+            v.set_long(cmpImpl(v.ul, other.ul));
+        else
+            v.set_long(cmpImpl(v.l, other.l));
+        return v;
+    }
+
 #define PP_DEFINE_BIN_OP(name, op) \
     inline Value operator op(const Value &other) const \
     { \
@@ -251,7 +255,6 @@ struct Value
 };
 
 } // namespace Internal
-} // namespace CPlusPlus
 
 using namespace CPlusPlus;
 using namespace CPlusPlus::Internal;
@@ -267,7 +270,7 @@ Macro *macroDefinition(const ByteArrayRef &name,
                        unsigned bytesOffset,
                        unsigned utf16charsOffset,
                        unsigned line,
-                       Environment *env,
+                       CPlusPlus::Environment *env,
                        Client *client)
 {
     Macro *m = env->resolve(name);
@@ -336,8 +339,8 @@ class ExpressionEvaluator
     void operator = (const ExpressionEvaluator &other);
 
 public:
-    ExpressionEvaluator(Client *client, Environment *env)
-        : client(client), env(env), _lex(0)
+    ExpressionEvaluator(Client *client, CPlusPlus::Environment *env)
+        : client(client), env(env), _lex(nullptr)
     { }
 
     Value operator()(const Token *firstToken, const Token *lastToken,
@@ -416,7 +419,7 @@ protected:
                                                 (*_lex)->byteOffset,
                                                 (*_lex)->utf16charOffset,
                                                 (*_lex)->lineno, env, client)
-                                != 0);
+                                != nullptr);
                 ++(*_lex);
             } else if ((*_lex)->is(T_LPAREN)) {
                 ++(*_lex);
@@ -426,7 +429,7 @@ protected:
                                                     (*_lex)->utf16charOffset,
                                                     (*_lex)->lineno,
                                                     env, client)
-                                    != 0);
+                                    != nullptr);
                     ++(*_lex);
                     if ((*_lex)->is(T_RPAREN))
                         ++(*_lex);
@@ -504,24 +507,25 @@ private:
     inline int precedence(int tokenKind) const
     {
         switch (tokenKind) {
-        case T_PIPE_PIPE:       return 0;
-        case T_AMPER_AMPER:     return 1;
-        case T_PIPE:            return 2;
-        case T_CARET:           return 3;
-        case T_AMPER:           return 4;
+        case T_PIPE_PIPE:           return 0;
+        case T_AMPER_AMPER:         return 1;
+        case T_PIPE:                return 2;
+        case T_CARET:               return 3;
+        case T_AMPER:               return 4;
         case T_EQUAL_EQUAL:
-        case T_EXCLAIM_EQUAL:   return 5;
+        case T_EXCLAIM_EQUAL:       return 5;
         case T_GREATER:
         case T_LESS:
         case T_LESS_EQUAL:
-        case T_GREATER_EQUAL:   return 6;
+        case T_GREATER_EQUAL:       return 6;
+        case T_LESS_EQUAL_GREATER:  return 7;
         case T_LESS_LESS:
-        case T_GREATER_GREATER: return 7;
+        case T_GREATER_GREATER:     return 8;
         case T_PLUS:
-        case T_MINUS:           return 8;
+        case T_MINUS:               return 9;
         case T_STAR:
         case T_SLASH:
-        case T_PERCENT:         return 9;
+        case T_PERCENT:             return 10;
 
         default:
             return -1;
@@ -541,6 +545,7 @@ private:
         case T_GREATER:
         case T_LESS:
         case T_LESS_EQUAL:
+        case T_LESS_EQUAL_GREATER:
         case T_GREATER_EQUAL:
         case T_LESS_LESS:
         case T_GREATER_GREATER:
@@ -559,24 +564,25 @@ private:
     static inline Value evaluate_expression(int tokenKind, const Value &lhs, const Value &rhs)
     {
         switch (tokenKind) {
-        case T_PIPE_PIPE:       return lhs || rhs;
-        case T_AMPER_AMPER:     return lhs && rhs;
-        case T_PIPE:            return lhs | rhs;
-        case T_CARET:           return lhs ^ rhs;
-        case T_AMPER:           return lhs & rhs;
-        case T_EQUAL_EQUAL:     return lhs == rhs;
-        case T_EXCLAIM_EQUAL:   return lhs != rhs;
-        case T_GREATER:         return lhs > rhs;
-        case T_LESS:            return lhs < rhs;
-        case T_LESS_EQUAL:      return lhs <= rhs;
-        case T_GREATER_EQUAL:   return lhs >= rhs;
-        case T_LESS_LESS:       return lhs << rhs;
-        case T_GREATER_GREATER: return lhs >> rhs;
-        case T_PLUS:            return lhs + rhs;
-        case T_MINUS:           return lhs - rhs;
-        case T_STAR:            return lhs * rhs;
-        case T_SLASH:           return rhs.is_zero() ? Value() : lhs / rhs;
-        case T_PERCENT:         return rhs.is_zero() ? Value() : lhs % rhs;
+        case T_PIPE_PIPE:            return lhs || rhs;
+        case T_AMPER_AMPER:          return lhs && rhs;
+        case T_PIPE:                 return lhs | rhs;
+        case T_CARET:                return lhs ^ rhs;
+        case T_AMPER:                return lhs & rhs;
+        case T_EQUAL_EQUAL:          return lhs == rhs;
+        case T_EXCLAIM_EQUAL:        return lhs != rhs;
+        case T_GREATER:              return lhs > rhs;
+        case T_LESS:                 return lhs < rhs;
+        case T_LESS_EQUAL:           return lhs <= rhs;
+        case T_LESS_EQUAL_GREATER:   return lhs.cmp(rhs);
+        case T_GREATER_EQUAL:        return lhs >= rhs;
+        case T_LESS_LESS:            return lhs << rhs;
+        case T_GREATER_GREATER:      return lhs >> rhs;
+        case T_PLUS:                 return lhs + rhs;
+        case T_MINUS:                return lhs - rhs;
+        case T_STAR:                 return lhs * rhs;
+        case T_SLASH:                return rhs.is_zero() ? Value() : lhs / rhs;
+        case T_PERCENT:              return rhs.is_zero() ? Value() : lhs % rhs;
 
         default:
             return Value();
@@ -594,21 +600,21 @@ private:
 } // end of anonymous namespace
 
 Preprocessor::State::State()
-    : m_lexer(0)
+    : m_lexer(nullptr)
     , m_skipping(MAX_LEVEL)
     , m_trueTest(MAX_LEVEL)
     , m_ifLevel(0)
     , m_tokenBufferDepth(0)
-    , m_tokenBuffer(0)
+    , m_tokenBuffer(nullptr)
     , m_inPreprocessorDirective(false)
     , m_markExpandedTokens(true)
     , m_noLines(false)
     , m_inCondition(false)
     , m_bytesOffsetRef(0)
     , m_utf16charsOffsetRef(0)
-    , m_result(0)
+    , m_result(nullptr)
     , m_lineRef(1)
-    , m_currentExpansion(0)
+    , m_currentExpansion(nullptr)
     , m_includeGuardState(IncludeGuardState_BeforeIfndef)
 {
     m_skipping[m_ifLevel] = false;
@@ -732,7 +738,11 @@ void Preprocessor::State::updateIncludeGuardState_helper(IncludeGuardStateHint h
 #endif // DEBUG_INCLUDE_GUARD_TRACKING
 }
 
-QString Preprocessor::configurationFileName() { return QStringLiteral("<configuration>"); }
+const FilePath &Preprocessor::configurationFileName()
+{
+    const static FilePath configurationFile = FilePath::fromPathPart(u"<configuration>");
+    return configurationFile;
+}
 
 Preprocessor::Preprocessor(Client *client, Environment *env)
     : m_client(client)
@@ -742,9 +752,12 @@ Preprocessor::Preprocessor(Client *client, Environment *env)
 {
 }
 
-QByteArray Preprocessor::run(const QString &fileName, const QString &source)
+QByteArray Preprocessor::run(const Utils::FilePath &filePath,
+                             const QByteArray &source,
+                             bool noLines,
+                             bool markGeneratedTokens)
 {
-    return run(fileName, source.toUtf8());
+    return run(filePath.toString(), source, noLines, markGeneratedTokens);
 }
 
 QByteArray Preprocessor::run(const QString &fileName,
@@ -758,7 +771,7 @@ QByteArray Preprocessor::run(const QString &fileName,
     preprocessed.reserve(source.size() * 2); // multiply by 2 because we insert #gen lines.
     preprocess(fileName, source, &preprocessed, &includeGuardMacroName, noLines,
                markGeneratedTokens, false);
-    if (!includeGuardMacroName.isEmpty())
+    if (m_client && !includeGuardMacroName.isEmpty())
         m_client->markAsIncludeGuard(includeGuardMacroName);
     return preprocessed;
 }
@@ -842,7 +855,7 @@ void Preprocessor::handleDefined(PPToken *tk)
 void Preprocessor::pushToken(Preprocessor::PPToken *tk)
 {
     const PPToken currentTokenBuffer[] = {*tk};
-    m_state.pushTokenBuffer(currentTokenBuffer, currentTokenBuffer + 1, 0);
+    m_state.pushTokenBuffer(currentTokenBuffer, currentTokenBuffer + 1, nullptr);
 }
 
 void Preprocessor::lex(PPToken *tk)
@@ -956,9 +969,7 @@ bool Preprocessor::handleIdentifier(PPToken *tk)
 
     Macro *macro = m_env->resolve(macroNameRef);
     if (!macro
-            || (tk->expanded()
-                && m_state.m_tokenBuffer
-                && m_state.m_tokenBuffer->isBlocked(macro))) {
+        || (tk->expanded() && m_state.m_tokenBuffer && m_state.m_tokenBuffer->isBlocked(macro))) {
         return false;
     }
 //    qDebug() << "expanding" << macro->name() << "on line" << tk->lineno;
@@ -982,16 +993,18 @@ bool Preprocessor::handleIdentifier(PPToken *tk)
         if (!expandFunctionlikeMacros()
                 // Still expand if this originally started with an object-like macro.
                 && m_state.m_expansionStatus != Expanding) {
-            m_client->notifyMacroReference(m_state.m_bytesOffsetRef + idTk.byteOffset,
-                                           m_state.m_utf16charsOffsetRef + idTk.utf16charOffset,
-                                           idTk.lineno,
-                                           *macro);
+            if (m_client) {
+                m_client->notifyMacroReference(m_state.m_bytesOffsetRef + idTk.byteOffset,
+                                               m_state.m_utf16charsOffsetRef + idTk.utf16charOffset,
+                                               idTk.lineno,
+                                               *macro);
+            }
             return false;
         }
 
         // Collect individual tokens that form the macro arguments.
         QVector<QVector<PPToken> > allArgTks;
-        bool hasArgs = collectActualArguments(tk, &allArgTks);
+        bool hasArgs = collectActualArguments(tk, &allArgTks, macro->name());
 
         // Check whether collecting arguments failed due to a previously added marker
         // that goot nested in a sequence of expansions. If so, store it and try again.
@@ -1001,7 +1014,7 @@ bool Preprocessor::handleIdentifier(PPToken *tk)
                 && (m_state.m_expansionStatus == Expanding
                     || m_state.m_expansionStatus == ReadyForExpansion)) {
             oldMarkerTk = *tk;
-            hasArgs = collectActualArguments(tk, &allArgTks);
+            hasArgs = collectActualArguments(tk, &allArgTks, macro->name());
         }
 
         // Check for matching parameter/argument count.
@@ -1127,7 +1140,8 @@ bool Preprocessor::handleIdentifier(PPToken *tk)
         }
     }
 
-    m_state.pushTokenBuffer(body.begin(), body.end(), macro);
+    const PPToken *start = body.constData();
+    m_state.pushTokenBuffer(start, start + body.size(), macro);
 
     if (m_client && !idTk.generated())
         m_client->stopExpandingMacro(idTk.byteOffset, *macro);
@@ -1141,7 +1155,12 @@ bool Preprocessor::handleFunctionLikeMacro(const Macro *macro,
                                            unsigned baseLine)
 {
     QVector<PPToken> expanded;
-    expanded.reserve(MAX_TOKEN_EXPANSION_COUNT);
+
+    const auto addToken = [&expanded](PPToken &&tok) {
+        if (expanded.isEmpty())
+            expanded.reserve(50);
+        expanded.push_back(std::move(tok));
+    };
 
     const size_t bodySize = body.size();
     for (size_t i = 0; i < bodySize && expanded.size() < MAX_TOKEN_EXPANSION_COUNT;
@@ -1183,10 +1202,10 @@ bool Preprocessor::handleFunctionLikeMacro(const Macro *macro,
                         enclosedString.replace("\\", "\\\\");
                         enclosedString.replace("\"", "\\\"");
 
-                        expanded.push_back(generateToken(T_STRING_LITERAL,
-                                                         enclosedString.constData(),
-                                                         enclosedString.size(),
-                                                         lineno, true));
+                        addToken(generateToken(T_STRING_LITERAL,
+                                               enclosedString.constData(),
+                                               enclosedString.size(),
+                                               lineno, true));
                     } else {
                         for (int k = 0; k < actualsSize; ++k) {
                             // Mark the actual tokens (which are the replaced version of the
@@ -1196,9 +1215,9 @@ bool Preprocessor::handleFunctionLikeMacro(const Macro *macro,
                             actual.f.expanded = true;
                             if (k == 0)
                                 actual.f.whitespace = bodyTk.whitespace();
-                            expanded += actual;
                             if (k == actualsSize - 1)
                                 lineno = actual.lineno;
+                            addToken(std::move(actual));
                         }
                     }
 
@@ -1213,12 +1232,12 @@ bool Preprocessor::handleFunctionLikeMacro(const Macro *macro,
                 // No formal macro parameter for this identifier in the body.
                 bodyTk.f.generated = true;
                 bodyTk.lineno = baseLine;
-                expanded.push_back(bodyTk);
+                addToken(std::move(bodyTk));
             }
         } else if (bodyTk.isNot(T_POUND) && bodyTk.isNot(T_POUND_POUND)) {
             bodyTk.f.generated = true;
             bodyTk.lineno = baseLine;
-            expanded.push_back(bodyTk);
+            addToken(std::move(bodyTk));
         }
 
         if (i > 1 && body[int(i) - 1].is(T_POUND_POUND)) {
@@ -1315,7 +1334,7 @@ void Preprocessor::trackExpansionCycles(PPToken *tk)
     }
 }
 
-static void adjustForCommentOrStringNewlines(unsigned *currentLine, const PPToken &tk)
+static void adjustForCommentOrStringNewlines(int *currentLine, const PPToken &tk)
 {
     if (tk.isComment() || tk.isStringLiteral())
         (*currentLine) += tk.asByteArrayRef().count('\n');
@@ -1337,7 +1356,7 @@ void Preprocessor::synchronizeOutputLines(const PPToken &tk, bool forceLine)
             generateOutputLineMarker(tk.lineno);
         }
     } else {
-        for (unsigned i = m_env->currentLine; i < tk.lineno; ++i)
+        for (int i = m_env->currentLine; i < tk.lineno; ++i)
             currentOutputBuffer().append('\n');
     }
 
@@ -1413,7 +1432,7 @@ void Preprocessor::preprocess(const QString &fileName, const QByteArray &source,
 
     ScopedSwap<QString> savedFileName(m_env->currentFile, fileName);
     ScopedSwap<QByteArray> savedUtf8FileName(m_env->currentFileUtf8, fileName.toUtf8());
-    ScopedSwap<unsigned> savedCurrentLine(m_env->currentLine, 1);
+    ScopedSwap<int> savedCurrentLine(m_env->currentLine, 1);
 
     if (!m_state.m_noLines)
         generateOutputLineMarker(1);
@@ -1436,7 +1455,7 @@ void Preprocessor::preprocess(const QString &fileName, const QByteArray &source,
                 trackedLine = tk.lineno;
                 trackedColumn = unsigned(computeDistance(tk, true));
             }
-            m_state.m_expandedTokensInfo.append(qMakePair(trackedLine, trackedColumn));
+            m_state.m_expandedTokensInfo.push_back({trackedLine, trackedColumn});
         } else if (m_state.m_expansionStatus == JustFinishedExpansion) {
             m_state.setExpansionStatus(NotExpanding);
             macroExpanded = true;
@@ -1498,11 +1517,21 @@ bool Preprocessor::consumeComments(PPToken *tk)
     return tk->isNot(T_EOF_SYMBOL);
 }
 
-bool Preprocessor::collectActualArguments(PPToken *tk, QVector<QVector<PPToken> > *actuals)
+bool Preprocessor::collectActualArguments(PPToken *tk, QVector<QVector<PPToken> > *actuals,
+                                          const QByteArray &parentMacroName)
 {
     Q_ASSERT(tk);
     Q_ASSERT(actuals);
 
+    ExecuteOnDestruction removeBlockedName;
+    if (m_state.m_tokenBuffer) {
+        removeBlockedName.reset([this] {
+            if (m_state.m_tokenBuffer && !m_state.m_tokenBuffer->blockedMacroNames.empty())
+                m_state.m_tokenBuffer->blockedMacroNames.pop_back();
+        });
+        m_state.m_tokenBuffer->blockedMacroNames.push_back(parentMacroName);
+    }
+
     lex(tk); // consume the identifier
 
     bool lastCommentIsCpp = false;
@@ -1541,7 +1570,8 @@ bool Preprocessor::collectActualArguments(PPToken *tk, QVector<QVector<PPToken>
     }
 
     if (!tk->is(T_RPAREN)) {
-        //###TODO: else error message
+        return false;
+        //###TODO: error message
     }
     return true;
 }
@@ -1648,6 +1678,15 @@ void Preprocessor::handleIncludeDirective(PPToken *tk, bool includeNext)
     if (m_cancelChecker && m_cancelChecker())
         return;
 
+    GuardLocker depthLocker(m_includeDepthGuard);
+    if (m_includeDepthGuard.lockCount() > MAX_INCLUDE_DEPTH) {
+        // FIXME: Categorized logging!
+#ifndef NO_DEBUG
+        std::cerr << "Maximum include depth exceeded" << m_state.m_currentFileName << std::endl;
+#endif
+        return;
+    }
+
     m_state.m_lexer->setScanAngleStringLiteralTokens(true);
     lex(tk); // consume "include" token
     m_state.m_lexer->setScanAngleStringLiteralTokens(false);
@@ -1680,7 +1719,7 @@ void Preprocessor::handleIncludeDirective(PPToken *tk, bool includeNext)
 
     if (m_client) {
         QString inc = QString::fromUtf8(included.constData() + 1, included.size() - 2);
-        m_client->sourceNeeded(line, inc, mode);
+        m_client->sourceNeeded(line, FilePath::fromString(inc), mode);
     }
 }
 
@@ -1696,7 +1735,7 @@ void Preprocessor::handleDefineDirective(PPToken *tk)
         return;
 
     Macro macro;
-    macro.setFileName(m_env->currentFile);
+    macro.setFilePath(FilePath::fromString(m_env->currentFile));
     macro.setLine(tk->lineno);
     QByteArray macroName = tk->asByteArrayRef().toByteArray();
     macro.setName(macroName);
@@ -1758,7 +1797,7 @@ void Preprocessor::handleDefineDirective(PPToken *tk)
     unsigned previousBytesOffset = 0;
     unsigned previousUtf16charsOffset = 0;
     unsigned previousLine = 0;
-    Macro *macroReference = 0;
+    Macro *macroReference = nullptr;
     while (isContinuationToken(*tk)) {
         // Macro tokens are always marked as expanded. However, only for object-like macros
         // we mark them as generated too. For function-like macros we postpone it until the
@@ -1775,15 +1814,15 @@ void Preprocessor::handleDefineDirective(PPToken *tk)
                 if (!macroReference->isFunctionLike()) {
                     m_client->notifyMacroReference(tk->byteOffset, tk->utf16charOffset,
                                                    tk->lineno, *macroReference);
-                    macroReference = 0;
+                    macroReference = nullptr;
                 }
             }
         } else if (macroReference) {
-            if (tk->is(T_LPAREN)) {
+            if (m_client && tk->is(T_LPAREN)) {
                 m_client->notifyMacroReference(previousBytesOffset, previousUtf16charsOffset,
                                                previousLine, *macroReference);
             }
-            macroReference = 0;
+            macroReference = nullptr;
         }
 
         previousBytesOffset = tk->byteOffset;
@@ -1802,7 +1841,8 @@ void Preprocessor::handleDefineDirective(PPToken *tk)
         if (macro.isFunctionLike()) {
             macroId += '(';
             bool fst = true;
-            foreach (const QByteArray &formal, macro.formals()) {
+            const QVector<QByteArray> formals = macro.formals();
+            for (const QByteArray &formal : formals) {
                 if (! fst)
                     macroId += ", ";
                 fst = false;
@@ -1841,6 +1881,7 @@ QByteArray Preprocessor::expand(PPToken *tk, PPToken *lastConditionToken)
 {
     unsigned line = tk->lineno;
     unsigned bytesBegin = tk->bytesBegin();
+    const int originalOffset = tk->originalOffset();
     unsigned utf16charsBegin = tk->utf16charsBegin();
     PPToken lastTk;
     while (isContinuationToken(*tk)) {
@@ -1848,13 +1889,13 @@ QByteArray Preprocessor::expand(PPToken *tk, PPToken *lastConditionToken)
         lex(tk);
     }
     // Gather the exact spelling of the content in the source.
-    QByteArray condition(m_state.m_source.mid(bytesBegin, lastTk.bytesBegin() + lastTk.bytes()
-                                              - bytesBegin));
+    QByteArray condition(m_state.m_source.mid(originalOffset, lastTk.originalOffset() + lastTk.bytes()
+                                              - originalOffset));
 
 //    qDebug("*** Condition before: [%s]", condition.constData());
     QByteArray result;
     result.reserve(256);
-    preprocess(m_state.m_currentFileName, condition, &result, 0, true, false, true,
+    preprocess(m_state.m_currentFileName, condition, &result, nullptr, true, false, true,
                bytesBegin, utf16charsBegin, line);
     result.squeeze();
 //    qDebug("*** Condition after: [%s]", result.constData());
@@ -2005,13 +2046,13 @@ void Preprocessor::handleIfDefDirective(bool checkUndefined, PPToken *tk)
 
             // the macro is a feature constraint(e.g. QT_NO_XXX)
             if (checkUndefined && macroName.startsWith("QT_NO_")) {
-                if (macro->fileName() == configurationFileName()) {
+                if (macro->filePath() == configurationFileName()) {
                     // and it' defined in a pro file (e.g. DEFINES += QT_NO_QOBJECT)
 
                     value = false; // take the branch
                 }
             }
-        } else if (m_env->isBuiltinMacro(macroName)) {
+        } else if (Environment::isBuiltinMacro(macroName)) {
             value = true;
         }
 
@@ -2161,3 +2202,5 @@ void Preprocessor::maybeStartOutputLine()
     if (*ch == '\\')
         buffer.append('\n');
 }
+
+} // namespace CPlusPlus
