@@ -1,27 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "jsoncheck.h"
 
@@ -30,7 +8,7 @@
 
 #include <QDebug>
 #include <QLatin1String>
-#include <QRegExp>
+#include <QRegularExpression>
 
 #include <cmath>
 
@@ -84,6 +62,12 @@ void JsonCheck::postVisit(Node *)
     analysis()->m_ranking += previous.m_ranking;
 }
 
+bool JsonCheck::visit(AST::TemplateLiteral *ast)
+{
+    Node::accept(ast->expression, this);
+    return true;
+}
+
 bool JsonCheck::visit(ObjectPattern *ast)
 {
     if (!proceedCheck(JsonValue::Object, ast->lbraceToken))
@@ -123,7 +107,7 @@ bool JsonCheck::visit(ObjectPattern *ast)
     }
 
     QStringList missing;
-    foreach (const QString &property, properties) {
+    for (const QString &property : properties) {
         if (!propertiesFound.contains(property)) {
             m_schema->enterNestedPropertySchema(property);
             if (m_schema->required())
@@ -269,12 +253,12 @@ bool JsonCheck::visit(StringLiteral *ast)
 
     analysis()->boostRanking();
 
-    const QStringRef literal = ast->value;
+    const QStringView literal = ast->value;
 
     const QString &pattern = m_schema->pattern();
     if (!pattern.isEmpty()) {
-        QRegExp regExp(pattern);
-        if (regExp.indexIn(literal.toString()) == -1) {
+        const QRegularExpression regExp(pattern);
+        if (regExp.match(literal.toString()).hasMatch()) {
             analysis()->m_messages.append(Message(ErrInvalidStringValuePattern,
                                                   ast->firstSourceLocation(),
                                                   QString(), QString(), false));
@@ -310,6 +294,14 @@ bool JsonCheck::visit(StringLiteral *ast)
     return false;
 }
 
+void JsonCheck::throwRecursionDepthError()
+{
+    analysis()->m_messages.append(Message(ErrHitMaximumRecursion,
+                                          SourceLocation(),
+                                          QString(), QString(), false));
+
+}
+
 static QString formatExpectedTypes(QStringList all)
 {
     all.removeDuplicates();
