@@ -1,33 +1,13 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Tim Sander <tim@krieglstein.org>
-** Copyright (C) 2016 Denis Shienkov <denis.shienkov@gmail.com>
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Tim Sander <tim@krieglstein.org>
+// Copyright (C) 2016 Denis Shienkov <denis.shienkov@gmail.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+
+#include "baremetaldevice.h"
 
 #include "baremetalconstants.h"
-#include "baremetaldevice.h"
 #include "baremetaldeviceconfigurationwidget.h"
 #include "baremetaldeviceconfigurationwizard.h"
+#include "baremetaltr.h"
 #include "debugserverprovidermanager.h"
 #include "idebugserverprovider.h"
 
@@ -35,8 +15,7 @@
 
 using namespace ProjectExplorer;
 
-namespace BareMetal {
-namespace Internal {
+namespace BareMetal::Internal {
 
 const char debugServerProviderIdKeyC[] = "IDebugServerProviderId";
 
@@ -44,7 +23,7 @@ const char debugServerProviderIdKeyC[] = "IDebugServerProviderId";
 
 BareMetalDevice::BareMetalDevice()
 {
-    setDisplayType(tr("Bare Metal"));
+    setDisplayType(Tr::tr("Bare Metal"));
     setDefaultDisplayName(defaultDisplayName());
     setOsType(Utils::OsTypeOther);
 }
@@ -58,7 +37,7 @@ BareMetalDevice::~BareMetalDevice()
 
 QString BareMetalDevice::defaultDisplayName()
 {
-    return tr("Bare Metal Device");
+    return Tr::tr("Bare Metal Device");
 }
 
 QString BareMetalDevice::debugServerProviderId() const
@@ -107,11 +86,6 @@ QVariantMap BareMetalDevice::toMap() const
     return map;
 }
 
-DeviceProcessSignalOperation::Ptr BareMetalDevice::signalOperation() const
-{
-    return DeviceProcessSignalOperation::Ptr();
-}
-
 IDeviceWidget *BareMetalDevice::createWidget()
 {
     return new BareMetalDeviceConfigurationWidget(sharedFromThis());
@@ -122,7 +96,7 @@ IDeviceWidget *BareMetalDevice::createWidget()
 BareMetalDeviceFactory::BareMetalDeviceFactory()
     : IDeviceFactory(Constants::BareMetalOsType)
 {
-    setDisplayName(BareMetalDevice::tr("Bare Metal Device"));
+    setDisplayName(Tr::tr("Bare Metal Device"));
     setCombinedIcon(":/baremetal/images/baremetaldevicesmall.png",
                     ":/baremetal/images/baremetaldevice.png");
     setConstructionFunction(&BareMetalDevice::create);
@@ -134,5 +108,4 @@ BareMetalDeviceFactory::BareMetalDeviceFactory()
     });
 }
 
-} //namespace Internal
-} //namespace BareMetal
+} // BareMetal::Internal
