@@ -1,68 +1,46 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Brian McGillion
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Brian McGillion
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "mercurialplugin.h"
-#include "optionspage.h"
+
+#include "commiteditor.h"
 #include "constants.h"
 #include "mercurialclient.h"
 #include "mercurialeditor.h"
+#include "mercurialsettings.h"
+#include "mercurialtr.h"
 #include "revertdialog.h"
 #include "srcdestdialog.h"
-#include "commiteditor.h"
-#include "mercurialsettings.h"
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
-#include <coreplugin/documentmanager.h>
-#include <coreplugin/id.h>
-#include <coreplugin/vcsmanager.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/documentmanager.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
-#include <coreplugin/editormanager/editormanager.h>
-
 #include <coreplugin/locator/commandlocator.h>
+#include <coreplugin/vcsmanager.h>
 
+#include <utils/commandline.h>
+#include <utils/environment.h>
 #include <utils/parameteraction.h>
 #include <utils/qtcassert.h>
 
 #include <vcsbase/basevcseditorfactory.h>
 #include <vcsbase/basevcssubmiteditorfactory.h>
-#include <vcsbase/vcsbaseeditor.h>
 #include <vcsbase/vcsbaseconstants.h>
-#include <vcsbase/vcsoutputwindow.h>
+#include <vcsbase/vcsbaseeditor.h>
 #include <vcsbase/vcscommand.h>
+#include <vcsbase/vcsoutputwindow.h>
 
 #include <QAction>
-#include <QMenu>
 #include <QDebug>
-#include <QtGlobal>
 #include <QDir>
-#include <QDialog>
 #include <QFileDialog>
+#include <QMenu>
+#include <QtGlobal>
 
 #ifdef WITH_TESTS
 #include <QTest>
@@ -72,8 +50,7 @@ using namespace VcsBase;
 using namespace Utils;
 using namespace std::placeholders;
 
-namespace Mercurial {
-namespace Internal {
+namespace Mercurial::Internal {
 
 class MercurialTopicCache : public Core::IVersionControl::TopicCache
 {
@@ -81,14 +58,14 @@ public:
     MercurialTopicCache(MercurialClient *client) : m_client(client) {}
 
 protected:
-    QString trackFile(const QString &repository) override
+    FilePath trackFile(const FilePath &repository) override
     {
-        return repository + QLatin1String("/.hg/branch");
+        return repository.pathAppended(".hg/branch");
     }
 
-    QString refreshTopic(const QString &repository) override
+    QString refreshTopic(const FilePath &repository) override
     {
-        return m_client->branchQuerySync(repository);
+        return m_client->branchQuerySync(repository.toString());
     }
 
 private:
@@ -125,31 +102,30 @@ const VcsBaseSubmitEditorParameters submitEditorParameters {
 
 class MercurialPluginPrivate final : public VcsBase::VcsBasePluginPrivate
 {
-    Q_DECLARE_TR_FUNCTIONS(Mercurial::Internal::MercurialPlugin)
-
 public:
     MercurialPluginPrivate();
 
     // IVersionControl
     QString displayName() const final;
-    Core::Id id() const final;
-    bool isVcsFileOrDirectory(const Utils::FilePath &fileName) const final;
+    Utils::Id id() const final;
+    bool isVcsFileOrDirectory(const FilePath &filePath) const final;
 
-    bool managesDirectory(const QString &filename, QString *topLevel = nullptr) const final;
-    bool managesFile(const QString &workingDirectory, const QString &fileName) const final;
+    bool managesDirectory(const FilePath &filePath, FilePath *topLevel = nullptr) const final;
+    bool managesFile(const FilePath &workingDirectory, const QString &fileName) const final;
     bool isConfigured() const final;
     bool supportsOperation(Operation operation) const final;
-    bool vcsOpen(const QString &fileName) final;
-    bool vcsAdd(const QString &filename) final;
-    bool vcsDelete(const QString &filename) final;
-    bool vcsMove(const QString &from, const QString &to) final;
-    bool vcsCreateRepository(const QString &directory) final;
-    bool vcsAnnotate(const QString &file, int line) final;
+    bool vcsOpen(const FilePath &filePath) final;
+    bool vcsAdd(const FilePath &filePath) final;
+    bool vcsDelete(const FilePath &filePath) final;
+    bool vcsMove(const FilePath &from, const FilePath &to) final;
+    bool vcsCreateRepository(const FilePath &directory) final;
+    void vcsAnnotate(const FilePath &filePath, int line) final;
+    void vcsDescribe(const FilePath &source, const QString &id) final { m_client.view(source, id); }
 
-    Core::ShellCommand *createInitialCheckoutCommand(const QString &url,
-                                                     const Utils::FilePath &baseDirectory,
-                                                     const QString &localName,
-                                                     const QStringList &extraArgs) final;
+    VcsCommand *createInitialCheckoutCommand(const QString &url,
+                                             const Utils::FilePath &baseDirectory,
+                                             const QString &localName,
+                                             const QStringList &extraArgs) final;
 
     bool sccManaged(const QString &filename);
 
@@ -160,7 +136,7 @@ public:
 
 private:
     void updateActions(VcsBase::VcsBasePluginPrivate::ActionState) final;
-    bool submitEditorAboutToClose() final;
+    bool activateCommit() final;
 
     // File menu action slots
     void addCurrentFile();
@@ -185,7 +161,6 @@ private:
     void outgoing();
     void commit();
     void showCommitWidget(const QList<VcsBase::VcsBaseClient::StatusItem> &status);
-    void commitFromEditor() override;
     void diffFromEditorSelected(const QStringList &files);
 
     void createMenu(const Core::Context &context);
@@ -193,13 +168,10 @@ private:
     void createDirectoryActions(const Core::Context &context);
     void createRepositoryActions(const Core::Context &context);
 
-    void describe(const QString &source, const QString &id) { m_client.view(source, id); }
-
     // Variables
     MercurialSettings m_settings;
     MercurialClient m_client{&m_settings};
-
-    OptionsPage m_optionsPage{[this] { configurationChanged(); }, &m_settings};
+    MercurialSettingsPage m_settingsPage{&m_settings};
 
     Core::CommandLocator *m_commandLocator = nullptr;
     Core::ActionContainer *m_mercurialContainer = nullptr;
@@ -218,9 +190,7 @@ private:
     QAction *m_createRepositoryAction = nullptr;
     QAction *m_menuAction = nullptr;
 
-    QString m_submitRepository;
-
-    bool m_submitActionTriggered = false;
+    FilePath m_submitRepository;
 
 public:
     VcsSubmitEditorFactory submitEditorFactory {
@@ -232,19 +202,19 @@ public:
     VcsEditorFactory logEditorFactory {
         &logEditorParameters,
         [this] { return new MercurialEditorWidget(&m_client); },
-        std::bind(&MercurialPluginPrivate::describe, this, _1, _2)
+        std::bind(&MercurialPluginPrivate::vcsDescribe, this, _1, _2)
     };
 
     VcsEditorFactory annotateEditorFactory {
         &annotateEditorParameters,
         [this] { return new MercurialEditorWidget(&m_client); },
-        std::bind(&MercurialPluginPrivate::describe, this, _1, _2)
+        std::bind(&MercurialPluginPrivate::vcsDescribe, this, _1, _2)
     };
 
     VcsEditorFactory diffEditorFactory {
         &diffEditorParameters,
         [this] { return new MercurialEditorWidget(&m_client); },
-        std::bind(&MercurialPluginPrivate::describe, this, _1, _2)
+        std::bind(&MercurialPluginPrivate::vcsDescribe, this, _1, _2)
     };
 };
 
@@ -256,10 +226,9 @@ MercurialPlugin::~MercurialPlugin()
     dd = nullptr;
 }
 
-bool MercurialPlugin::initialize(const QStringList & /* arguments */, QString * /*errorMessage */)
+void MercurialPlugin::initialize()
 {
     dd = new MercurialPluginPrivate;
-    return true;
 }
 
 void MercurialPlugin::extensionsInitialized()
@@ -281,8 +250,11 @@ MercurialPluginPrivate::MercurialPluginPrivate()
 
     const QString prefix = QLatin1String("hg");
     m_commandLocator = new Core::CommandLocator("Mercurial", prefix, prefix, this);
+    m_commandLocator->setDescription(Tr::tr("Triggers a Mercurial version control operation."));
 
     createMenu(context);
+
+    connect(&m_settings, &AspectContainer::applied, this, &IVersionControl::configurationChanged);
 }
 
 void MercurialPluginPrivate::createMenu(const Core::Context &context)
@@ -290,7 +262,7 @@ void MercurialPluginPrivate::createMenu(const Core::Context &context)
     // Create menu item for Mercurial
     m_mercurialContainer = Core::ActionManager::createMenu("Mercurial.MercurialMenu");
     QMenu *menu = m_mercurialContainer->menu();
-    menu->setTitle(tr("Me&rcurial"));
+    menu->setTitle(Tr::tr("Me&rcurial"));
 
     createFileActions(context);
     m_mercurialContainer->addSeparator(context);
@@ -300,7 +272,7 @@ void MercurialPluginPrivate::createMenu(const Core::Context &context)
     m_mercurialContainer->addSeparator(context);
 
     // Request the Tools menu and add the Mercurial menu to it
-    Core::ActionContainer *toolsMenu = Core::ActionManager::actionContainer(Core::Id(Core::Constants::M_TOOLS));
+    Core::ActionContainer *toolsMenu = Core::ActionManager::actionContainer(Utils::Id(Core::Constants::M_TOOLS));
     toolsMenu->addMenu(m_mercurialContainer);
     m_menuAction = m_mercurialContainer->menu()->menuAction();
 }
@@ -309,55 +281,55 @@ void MercurialPluginPrivate::createFileActions(const Core::Context &context)
 {
     Core::Command *command;
 
-    annotateFile = new ParameterAction(tr("Annotate Current File"), tr("Annotate \"%1\""), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(annotateFile, Core::Id(Constants::ANNOTATE), context);
+    annotateFile = new ParameterAction(Tr::tr("Annotate Current File"), Tr::tr("Annotate \"%1\""), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(annotateFile, Utils::Id(Constants::ANNOTATE), context);
     command->setAttribute(Core::Command::CA_UpdateText);
     connect(annotateFile, &QAction::triggered, this, &MercurialPluginPrivate::annotateCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    diffFile = new ParameterAction(tr("Diff Current File"), tr("Diff \"%1\""), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(diffFile, Core::Id(Constants::DIFF), context);
+    diffFile = new ParameterAction(Tr::tr("Diff Current File"), Tr::tr("Diff \"%1\""), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(diffFile, Utils::Id(Constants::DIFF), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+D") : tr("Alt+G,Alt+D")));
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? Tr::tr("Meta+H,Meta+D") : Tr::tr("Alt+G,Alt+D")));
     connect(diffFile, &QAction::triggered, this, &MercurialPluginPrivate::diffCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    logFile = new ParameterAction(tr("Log Current File"), tr("Log \"%1\""), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(logFile, Core::Id(Constants::LOG), context);
+    logFile = new ParameterAction(Tr::tr("Log Current File"), Tr::tr("Log \"%1\""), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(logFile, Utils::Id(Constants::LOG), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+L") : tr("Alt+G,Alt+L")));
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? Tr::tr("Meta+H,Meta+L") : Tr::tr("Alt+G,Alt+L")));
     connect(logFile, &QAction::triggered, this, &MercurialPluginPrivate::logCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    statusFile = new ParameterAction(tr("Status Current File"), tr("Status \"%1\""), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(statusFile, Core::Id(Constants::STATUS), context);
+    statusFile = new ParameterAction(Tr::tr("Status Current File"), Tr::tr("Status \"%1\""), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(statusFile, Utils::Id(Constants::STATUS), context);
     command->setAttribute(Core::Command::CA_UpdateText);
-    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+S") : tr("Alt+G,Alt+S")));
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? Tr::tr("Meta+H,Meta+S") : Tr::tr("Alt+G,Alt+S")));
     connect(statusFile, &QAction::triggered, this, &MercurialPluginPrivate::statusCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
     m_mercurialContainer->addSeparator(context);
 
-    m_addAction = new ParameterAction(tr("Add"), tr("Add \"%1\""), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(m_addAction, Core::Id(Constants::ADD), context);
+    m_addAction = new ParameterAction(Tr::tr("Add"), Tr::tr("Add \"%1\""), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(m_addAction, Utils::Id(Constants::ADD), context);
     command->setAttribute(Core::Command::CA_UpdateText);
     connect(m_addAction, &QAction::triggered, this, &MercurialPluginPrivate::addCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    m_deleteAction = new ParameterAction(tr("Delete..."), tr("Delete \"%1\"..."), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(m_deleteAction, Core::Id(Constants::DELETE), context);
+    m_deleteAction = new ParameterAction(Tr::tr("Delete..."), Tr::tr("Delete \"%1\"..."), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(m_deleteAction, Utils::Id(Constants::DELETE), context);
     command->setAttribute(Core::Command::CA_UpdateText);
     connect(m_deleteAction, &QAction::triggered, this, &MercurialPluginPrivate::promptToDeleteCurrentFile);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    revertFile = new ParameterAction(tr("Revert Current File..."), tr("Revert \"%1\"..."), ParameterAction::EnabledWithParameter, this);
-    command = Core::ActionManager::registerAction(revertFile, Core::Id(Constants::REVERT), context);
+    revertFile = new ParameterAction(Tr::tr("Revert Current File..."), Tr::tr("Revert \"%1\"..."), ParameterAction::EnabledWithParameter, this);
+    command = Core::ActionManager::registerAction(revertFile, Utils::Id(Constants::REVERT), context);
     command->setAttribute(Core::Command::CA_UpdateText);
     connect(revertFile, &QAction::triggered, this, &MercurialPluginPrivate::revertCurrentFile);
     m_mercurialContainer->addAction(command);
@@ -378,7 +350,7 @@ void MercurialPluginPrivate::annotateCurrentFile()
         currentLine = editor->currentLine();
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client.annotate(state.currentFileTopLevel(), state.relativeCurrentFile(), QString(), currentLine);
+    m_client.annotate(state.currentFileTopLevel(), state.relativeCurrentFile(), currentLine);
 }
 
 void MercurialPluginPrivate::diffCurrentFile()
@@ -392,8 +364,7 @@ void MercurialPluginPrivate::logCurrentFile()
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasFile(), return);
-    m_client.log(state.currentFileTopLevel(), QStringList(state.relativeCurrentFile()),
-                 QStringList(), true);
+    m_client.log(state.currentFileTopLevel(), QStringList(state.relativeCurrentFile()), {}, true);
 }
 
 void MercurialPluginPrivate::revertCurrentFile()
@@ -416,30 +387,30 @@ void MercurialPluginPrivate::statusCurrentFile()
 
 void MercurialPluginPrivate::createDirectoryActions(const Core::Context &context)
 {
-    auto action = new QAction(tr("Diff"), this);
+    auto action = new QAction(Tr::tr("Diff"), this);
     m_repositoryActionList.append(action);
-    Core::Command *command = Core::ActionManager::registerAction(action, Core::Id(Constants::DIFFMULTI), context);
+    Core::Command *command = Core::ActionManager::registerAction(action, Utils::Id(Constants::DIFFMULTI), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::diffRepository);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Log"), this);
+    action = new QAction(Tr::tr("Log"), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::LOGMULTI), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::LOGMULTI), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::logRepository);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Revert..."), this);
+    action = new QAction(Tr::tr("Revert..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::REVERTMULTI), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::REVERTMULTI), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::revertMulti);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Status"), this);
+    action = new QAction(Tr::tr("Status"), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::STATUSMULTI), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::STATUSMULTI), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::statusMulti);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
@@ -480,58 +451,58 @@ void MercurialPluginPrivate::statusMulti()
 
 void MercurialPluginPrivate::createRepositoryActions(const Core::Context &context)
 {
-    auto action = new QAction(tr("Pull..."), this);
+    auto action = new QAction(Tr::tr("Pull..."), this);
     m_repositoryActionList.append(action);
-    Core::Command *command = Core::ActionManager::registerAction(action, Core::Id(Constants::PULL), context);
+    Core::Command *command = Core::ActionManager::registerAction(action, Utils::Id(Constants::PULL), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::pull);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Push..."), this);
+    action = new QAction(Tr::tr("Push..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::PUSH), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::PUSH), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::push);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Update..."), this);
+    action = new QAction(Tr::tr("Update..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::UPDATE), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::UPDATE), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::update);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Import..."), this);
+    action = new QAction(Tr::tr("Import..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::IMPORT), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::IMPORT), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::import);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Incoming..."), this);
+    action = new QAction(Tr::tr("Incoming..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::INCOMING), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::INCOMING), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::incoming);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Outgoing..."), this);
+    action = new QAction(Tr::tr("Outgoing..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::OUTGOING), context);
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::OUTGOING), context);
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::outgoing);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    action = new QAction(tr("Commit..."), this);
+    action = new QAction(Tr::tr("Commit..."), this);
     m_repositoryActionList.append(action);
-    command = Core::ActionManager::registerAction(action, Core::Id(Constants::COMMIT), context);
-    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? tr("Meta+H,Meta+C") : tr("Alt+G,Alt+C")));
+    command = Core::ActionManager::registerAction(action, Utils::Id(Constants::COMMIT), context);
+    command->setDefaultKeySequence(QKeySequence(Core::useMacShortcuts ? Tr::tr("Meta+H,Meta+C") : Tr::tr("Alt+G,Alt+C")));
     connect(action, &QAction::triggered, this, &MercurialPluginPrivate::commit);
     m_mercurialContainer->addAction(command);
     m_commandLocator->appendCommand(command);
 
-    m_createRepositoryAction = new QAction(tr("Create Repository..."), this);
-    command = Core::ActionManager::registerAction(m_createRepositoryAction, Core::Id(Constants::CREATE_REPOSITORY), context);
+    m_createRepositoryAction = new QAction(Tr::tr("Create Repository..."), this);
+    command = Core::ActionManager::registerAction(m_createRepositoryAction, Utils::Id(Constants::CREATE_REPOSITORY), context);
     connect(m_createRepositoryAction, &QAction::triggered, this, &MercurialPluginPrivate::createRepository);
     m_mercurialContainer->addAction(command);
 }
@@ -542,7 +513,7 @@ void MercurialPluginPrivate::pull()
     QTC_ASSERT(state.hasTopLevel(), return);
 
     SrcDestDialog dialog(state, SrcDestDialog::incoming, Core::ICore::dialogParent());
-    dialog.setWindowTitle(tr("Pull Source"));
+    dialog.setWindowTitle(Tr::tr("Pull Source"));
     if (dialog.exec() != QDialog::Accepted)
         return;
     m_client.synchronousPull(dialog.workingDir(), dialog.getRepositoryString());
@@ -554,7 +525,7 @@ void MercurialPluginPrivate::push()
     QTC_ASSERT(state.hasTopLevel(), return);
 
     SrcDestDialog dialog(state, SrcDestDialog::outgoing, Core::ICore::dialogParent());
-    dialog.setWindowTitle(tr("Push Destination"));
+    dialog.setWindowTitle(Tr::tr("Push Destination"));
     if (dialog.exec() != QDialog::Accepted)
         return;
     m_client.synchronousPush(dialog.workingDir(), dialog.getRepositoryString());
@@ -566,7 +537,7 @@ void MercurialPluginPrivate::update()
     QTC_ASSERT(state.hasTopLevel(), return);
 
     RevertDialog updateDialog(Core::ICore::dialogParent());
-    updateDialog.setWindowTitle(tr("Update"));
+    updateDialog.setWindowTitle(Tr::tr("Update"));
     if (updateDialog.exec() != QDialog::Accepted)
         return;
     m_client.update(state.topLevel(), updateDialog.revision());
@@ -594,7 +565,7 @@ void MercurialPluginPrivate::incoming()
     QTC_ASSERT(state.hasTopLevel(), return);
 
     SrcDestDialog dialog(state, SrcDestDialog::incoming, Core::ICore::dialogParent());
-    dialog.setWindowTitle(tr("Incoming Source"));
+    dialog.setWindowTitle(Tr::tr("Incoming Source"));
     if (dialog.exec() != QDialog::Accepted)
         return;
     m_client.incoming(state.topLevel(), dialog.getRepositoryString());
@@ -630,7 +601,7 @@ void MercurialPluginPrivate::showCommitWidget(const QList<VcsBaseClient::StatusI
     disconnect(&m_client, &MercurialClient::parsedStatus, this, &MercurialPluginPrivate::showCommitWidget);
 
     if (status.isEmpty()) {
-        VcsOutputWindow::appendError(tr("There are no changes to commit."));
+        VcsOutputWindow::appendError(Tr::tr("There are no changes to commit."));
         return;
     }
 
@@ -643,10 +614,9 @@ void MercurialPluginPrivate::showCommitWidget(const QList<VcsBaseClient::StatusI
         return;
     }
 
-    Core::IEditor *editor = Core::EditorManager::openEditor(saver.fileName(),
-                                                            Constants::COMMIT_ID);
+    Core::IEditor *editor = Core::EditorManager::openEditor(saver.filePath(), Constants::COMMIT_ID);
     if (!editor) {
-        VcsOutputWindow::appendError(tr("Unable to create an editor for the commit."));
+        VcsOutputWindow::appendError(Tr::tr("Unable to create an editor for the commit."));
         return;
     }
 
@@ -658,14 +628,13 @@ void MercurialPluginPrivate::showCommitWidget(const QList<VcsBaseClient::StatusI
             this, &MercurialPluginPrivate::diffFromEditorSelected);
     commitEditor->setCheckScriptWorkingDirectory(m_submitRepository);
 
-    const QString msg = tr("Commit changes for \"%1\".").
-                        arg(QDir::toNativeSeparators(m_submitRepository));
+    const QString msg = Tr::tr("Commit changes for \"%1\".").arg(m_submitRepository.toUserOutput());
     commitEditor->document()->setPreferredDisplayName(msg);
 
     const QString branch = vcsTopic(m_submitRepository);
     commitEditor->setFields(m_submitRepository, branch,
-                            m_settings.stringValue(MercurialSettings::userNameKey),
-                            m_settings.stringValue(MercurialSettings::userEmailKey), status);
+                            m_settings.userName.value(),
+                            m_settings.userEmail.value(), status);
 }
 
 void MercurialPluginPrivate::diffFromEditorSelected(const QStringList &files)
@@ -673,34 +642,13 @@ void MercurialPluginPrivate::diffFromEditorSelected(const QStringList &files)
     m_client.diff(m_submitRepository, files);
 }
 
-void MercurialPluginPrivate::commitFromEditor()
-{
-    // Close the submit editor
-    m_submitActionTriggered = true;
-    QTC_ASSERT(submitEditor(), return);
-    Core::EditorManager::closeDocument(submitEditor()->document());
-}
-
-bool MercurialPluginPrivate::submitEditorAboutToClose()
+bool MercurialPluginPrivate::activateCommit()
 {
     auto commitEditor = qobject_cast<CommitEditor *>(submitEditor());
     QTC_ASSERT(commitEditor, return true);
     Core::IDocument *editorFile = commitEditor->document();
     QTC_ASSERT(editorFile, return true);
 
-    const VcsBaseSubmitEditor::PromptSubmitResult response =
-            commitEditor->promptSubmit(this, nullptr, !m_submitActionTriggered);
-    m_submitActionTriggered = false;
-
-    switch (response) {
-    case VcsBaseSubmitEditor::SubmitCanceled:
-        return false;
-    case VcsBaseSubmitEditor::SubmitDiscarded:
-        return true;
-    default:
-        break;
-    }
-
     const QStringList files = commitEditor->checkedFiles();
     if (!files.empty()) {
         //save the commit message
@@ -734,42 +682,41 @@ void MercurialPluginPrivate::updateActions(VcsBasePluginPrivate::ActionState as)
     revertFile->setParameter(filename);
     statusFile->setParameter(filename);
 
-    foreach (QAction *repoAction, m_repositoryActionList)
+    for (QAction *repoAction : std::as_const(m_repositoryActionList))
         repoAction->setEnabled(repoEnabled);
 }
 
 QString MercurialPluginPrivate::displayName() const
 {
-    return tr("Mercurial");
+    return Tr::tr("Mercurial");
 }
 
-Core::Id MercurialPluginPrivate::id() const
+Utils::Id MercurialPluginPrivate::id() const
 {
     return {VcsBase::Constants::VCS_ID_MERCURIAL};
 }
 
-bool MercurialPluginPrivate::isVcsFileOrDirectory(const Utils::FilePath &fileName) const
+bool MercurialPluginPrivate::isVcsFileOrDirectory(const FilePath &filePath) const
 {
-    return m_client.isVcsDirectory(fileName);
+    return m_client.isVcsDirectory(filePath);
 }
 
-bool MercurialPluginPrivate::managesDirectory(const QString &directory, QString *topLevel) const
+bool MercurialPluginPrivate::managesDirectory(const FilePath &filePath, FilePath *topLevel) const
 {
-    QFileInfo dir(directory);
-    const QString topLevelFound = m_client.findTopLevelForFile(dir);
+    const FilePath topLevelFound = m_client.findTopLevelForFile(filePath);
     if (topLevel)
         *topLevel = topLevelFound;
     return !topLevelFound.isEmpty();
 }
 
-bool MercurialPluginPrivate::managesFile(const QString &workingDirectory, const QString &fileName) const
+bool MercurialPluginPrivate::managesFile(const FilePath &workingDirectory, const QString &fileName) const
 {
     return m_client.managesFile(workingDirectory, fileName);
 }
 
 bool MercurialPluginPrivate::isConfigured() const
 {
-    const Utils::FilePath binary = m_settings.binaryPath();
+    const FilePath binary = m_settings.binaryPath.filePath();
     if (binary.isEmpty())
         return false;
     QFileInfo fi = binary.toFileInfo();
@@ -794,66 +741,61 @@ bool MercurialPluginPrivate::supportsOperation(Operation operation) const
     return supported;
 }
 
-bool MercurialPluginPrivate::vcsOpen(const QString &filename)
+bool MercurialPluginPrivate::vcsOpen(const FilePath &filePath)
 {
-    Q_UNUSED(filename)
+    Q_UNUSED(filePath)
     return true;
 }
 
-bool MercurialPluginPrivate::vcsAdd(const QString &filename)
+bool MercurialPluginPrivate::vcsAdd(const FilePath &filePath)
 {
-    const QFileInfo fi(filename);
-    return m_client.synchronousAdd(fi.absolutePath(), fi.fileName());
+    return m_client.synchronousAdd(filePath.parentDir(), filePath.fileName());
 }
 
-bool MercurialPluginPrivate::vcsDelete(const QString &filename)
+bool MercurialPluginPrivate::vcsDelete(const FilePath &filePath)
 {
-    const QFileInfo fi(filename);
-    return m_client.synchronousRemove(fi.absolutePath(), fi.fileName());
+    return m_client.synchronousRemove(filePath.parentDir(), filePath.fileName());
 }
 
-bool MercurialPluginPrivate::vcsMove(const QString &from, const QString &to)
+bool MercurialPluginPrivate::vcsMove(const FilePath &from, const FilePath &to)
 {
-    const QFileInfo fromInfo(from);
-    const QFileInfo toInfo(to);
-    return m_client.synchronousMove(fromInfo.absolutePath(),
-                                            fromInfo.absoluteFilePath(),
-                                            toInfo.absoluteFilePath());
+    const QFileInfo fromInfo = from.toFileInfo();
+    const QFileInfo toInfo = to.toFileInfo();
+    return m_client.synchronousMove(from.parentDir(),
+                                    fromInfo.absoluteFilePath(),
+                                    toInfo.absoluteFilePath());
 }
 
-bool MercurialPluginPrivate::vcsCreateRepository(const QString &directory)
+bool MercurialPluginPrivate::vcsCreateRepository(const FilePath &directory)
 {
     return m_client.synchronousCreateRepository(directory);
 }
 
-bool MercurialPluginPrivate::vcsAnnotate(const QString &file, int line)
+void MercurialPluginPrivate::vcsAnnotate(const FilePath &filePath, int line)
 {
-    const QFileInfo fi(file);
-    m_client.annotate(fi.absolutePath(), fi.fileName(), QString(), line);
-    return true;
+    m_client.annotate(filePath.parentDir(), filePath.fileName(), line);
 }
 
-Core::ShellCommand *MercurialPluginPrivate::createInitialCheckoutCommand(const QString &url,
-                                                                   const Utils::FilePath &baseDirectory,
-                                                                   const QString &localName,
-                                                                   const QStringList &extraArgs)
+VcsCommand *MercurialPluginPrivate::createInitialCheckoutCommand(const QString &url,
+                                                                 const Utils::FilePath &baseDirectory,
+                                                                 const QString &localName,
+                                                                 const QStringList &extraArgs)
 {
     QStringList args;
     args << QLatin1String("clone") << extraArgs << url << localName;
-    auto command = new VcsBase::VcsCommand(baseDirectory.toString(),
-                                           m_client.processEnvironment());
-    command->addJob({m_settings.binaryPath(), args}, -1);
+    auto command = VcsBaseClient::createVcsCommand(baseDirectory, m_client.processEnvironment());
+    command->addJob({m_settings.binaryPath.filePath(), args}, -1);
     return command;
 }
 
 bool MercurialPluginPrivate::sccManaged(const QString &filename)
 {
     const QFileInfo fi(filename);
-    QString topLevel;
-    const bool managed = managesDirectory(fi.absolutePath(), &topLevel);
+    FilePath topLevel;
+    const bool managed = managesDirectory(FilePath::fromString(fi.absolutePath()), &topLevel);
     if (!managed || topLevel.isEmpty())
         return false;
-    const QDir topLevelDir(topLevel);
+    const QDir topLevelDir(topLevel.toString());
     return m_client.manifestSync(topLevel, topLevelDir.relativeFilePath(filename));
 }
 
@@ -861,7 +803,7 @@ void MercurialPluginPrivate::changed(const QVariant &v)
 {
     switch (v.type()) {
     case QVariant::String:
-        emit repositoryChanged(v.toString());
+        emit repositoryChanged(FilePath::fromVariant(v));
         break;
     case QVariant::StringList:
         emit filesChanged(v.toStringList());
@@ -926,5 +868,4 @@ void MercurialPlugin::testLogResolving()
 }
 #endif
 
-} // namespace Internal
-} // namespace Mercurial
+} // Mercurial::Internal
