@@ -1,39 +1,22 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "idevice.h"
 
 #include "devicemanager.h"
 #include "deviceprocesslist.h"
 #include "idevicefactory.h"
+#include "sshparameters.h"
 
 #include "../kit.h"
 #include "../kitinformation.h"
-#include "../runconfiguration.h"
+#include "../projectexplorertr.h"
+#include "../target.h"
 
-#include <ssh/sshconnection.h>
+#include <coreplugin/icore.h>
+
+#include <utils/devicefileaccess.h>
+#include <utils/displayname.h>
 #include <utils/icon.h>
 #include <utils/portlist.h>
 #include <utils/qtcassert.h>
@@ -42,6 +25,8 @@
 #include <QCoreApplication>
 #include <QStandardPaths>
 
+#include <QDateTime>
+#include <QReadWriteLock>
 #include <QString>
 #include <QUuid>
 
@@ -69,7 +54,7 @@
  */
 
 /*!
- * \fn Core::Id ProjectExplorer::IDevice::invalidId()
+ * \fn Utils::Id ProjectExplorer::IDevice::invalidId()
  * A value that no device can ever have as its internal id.
  */
 
@@ -96,14 +81,15 @@
  * Creates an identical copy of a device object.
  */
 
-
-static Core::Id newId()
-{
-    return Core::Id::fromString(QUuid::createUuid().toString());
-}
+using namespace Utils;
 
 namespace ProjectExplorer {
 
+static Id newId()
+{
+    return Id::fromString(QUuid::createUuid().toString());
+}
+
 const char DisplayNameKey[] = "Name";
 const char TypeKey[] = "OsType";
 const char IdKey[] = "InternalId";
@@ -123,10 +109,10 @@ const char TimeoutKey[] = "Timeout";
 const char HostKeyCheckingKey[] = "HostKeyChecking";
 
 const char DebugServerKey[] = "DebugServerKey";
-const char QmlsceneKey[] = "QmlsceneKey";
+const char QmlRuntimeKey[] = "QmlsceneKey";
 
-using AuthType = QSsh::SshConnectionParameters::AuthenticationType;
-const AuthType DefaultAuthType = QSsh::SshConnectionParameters::AuthenticationTypeAll;
+using AuthType = SshParameters::AuthenticationType;
+const AuthType DefaultAuthType = SshParameters::AuthenticationTypeAll;
 const IDevice::MachineType DefaultMachineType = IDevice::Hardware;
 
 const int DefaultTimeout = 10;
@@ -137,20 +123,26 @@ class IDevicePrivate
 public:
     IDevicePrivate() = default;
 
-    QString displayName;
-    Core::Id type;
+    DisplayName displayName;
+    QString displayType;
+    Id type;
     IDevice::Origin origin = IDevice::AutoDetected;
-    Core::Id id;
+    Id id;
     IDevice::DeviceState deviceState = IDevice::DeviceStateUnknown;
     IDevice::MachineType machineType = IDevice::Hardware;
+    OsType osType = OsTypeOther;
+    DeviceFileAccess *fileAccess = nullptr;
     int version = 0; // This is used by devices that have been added by the SDK.
 
-    QSsh::SshConnectionParameters sshParameters;
-    Utils::PortList freePorts;
-    QString debugServerPath;
-    QString qmlsceneCommand;
+    QReadWriteLock lock; // Currently used to protect sshParameters only
+    SshParameters sshParameters;
+    PortList freePorts;
+    FilePath debugServerPath;
+    FilePath debugDumperPath = Core::ICore::resourcePath("debugger/");
+    FilePath qmlRunCommand;
+    bool emptyCommandAllowed = false;
 
-    QList<Utils::Icon> deviceIcons;
+    QList<Icon> deviceIcons;
     QList<IDevice::DeviceAction> deviceActions;
     QVariantMap extraData;
     IDevice::OpenTerminal openTerminal;
@@ -163,12 +155,14 @@ IDevice::IDevice() : d(new Internal::IDevicePrivate)
 {
 }
 
+IDevice::~IDevice() = default;
+
 void IDevice::setOpenTerminal(const IDevice::OpenTerminal &openTerminal)
 {
     d->openTerminal = openTerminal;
 }
 
-void IDevice::setupId(Origin origin, Core::Id id)
+void IDevice::setupId(Origin origin, Id id)
 {
     d->origin = origin;
     QTC_CHECK(origin == ManuallyAdded || id.isValid());
@@ -180,13 +174,87 @@ bool IDevice::canOpenTerminal() const
     return bool(d->openTerminal);
 }
 
-void IDevice::openTerminal(const Utils::Environment &env, const QString &workingDir) const
+void IDevice::openTerminal(const Environment &env, const FilePath &workingDir) const
 {
     QTC_ASSERT(canOpenTerminal(), return);
     d->openTerminal(env, workingDir);
 }
 
-IDevice::~IDevice() = default;
+bool IDevice::isEmptyCommandAllowed() const
+{
+    return d->emptyCommandAllowed;
+}
+
+void IDevice::setAllowEmptyCommand(bool allow)
+{
+    d->emptyCommandAllowed = allow;
+}
+
+bool IDevice::isAnyUnixDevice() const
+{
+    return d->osType == OsTypeLinux || d->osType == OsTypeMac || d->osType == OsTypeOtherUnix;
+}
+
+DeviceFileAccess *IDevice::fileAccess() const
+{
+    return d->fileAccess;
+}
+
+FilePath IDevice::filePath(const QString &pathOnDevice) const
+{
+    // match DeviceManager::deviceForPath
+    return FilePath::fromParts(u"device", id().toString(), pathOnDevice);
+}
+
+bool IDevice::handlesFile(const FilePath &filePath) const
+{
+    if (filePath.scheme() == u"device" && filePath.host() == id().toString())
+        return true;
+    return false;
+}
+
+FilePath IDevice::searchExecutableInPath(const QString &fileName) const
+{
+    FilePaths paths;
+    for (const FilePath &pathEntry : systemEnvironment().path())
+        paths.append(filePath(pathEntry.path()));
+    return searchExecutable(fileName, paths);
+}
+
+FilePath IDevice::searchExecutable(const QString &fileName, const FilePaths &dirs) const
+{
+    for (FilePath dir : dirs) {
+        if (!handlesFile(dir)) // Allow device-local dirs to be used.
+            dir = filePath(dir.path());
+        QTC_CHECK(handlesFile(dir));
+        const FilePath candidate = dir / fileName;
+        if (candidate.isExecutableFile())
+            return candidate;
+    }
+
+    return {};
+}
+
+ProcessInterface *IDevice::createProcessInterface() const
+{
+    QTC_CHECK(false);
+    return nullptr;
+}
+
+FileTransferInterface *IDevice::createFileTransferInterface(
+        const FileTransferSetupData &setup) const
+{
+    Q_UNUSED(setup)
+    QTC_CHECK(false);
+    return nullptr;
+}
+
+Environment IDevice::systemEnvironment() const
+{
+    DeviceFileAccess *access = fileAccess();
+    QTC_ASSERT(access, return Environment::systemEnvironment());
+    return access->deviceEnvironment();
+}
 
 /*!
     Specifies a free-text name for the device to be displayed in GUI elements.
@@ -194,19 +262,42 @@ IDevice::~IDevice() = default;
 
 QString IDevice::displayName() const
 {
-    return d->displayName;
+    return d->displayName.value();
 }
 
 void IDevice::setDisplayName(const QString &name)
 {
-    if (d->displayName == name)
-        return;
-    d->displayName = name;
+    d->displayName.setValue(name);
+}
+
+void IDevice::setDefaultDisplayName(const QString &name)
+{
+    d->displayName.setDefaultValue(name);
+}
+
+QString IDevice::displayType() const
+{
+    return d->displayType;
+}
+
+void IDevice::setDisplayType(const QString &type)
+{
+    d->displayType = type;
+}
+
+void IDevice::setOsType(OsType osType)
+{
+    d->osType = osType;
+}
+
+void IDevice::setFileAccess(DeviceFileAccess *fileAccess)
+{
+    d->fileAccess = fileAccess;
 }
 
 IDevice::DeviceInfo IDevice::deviceInformation() const
 {
-    const QString key = QCoreApplication::translate("ProjectExplorer::IDevice", "Device");
+    const QString key = Tr::tr("Device");
     return DeviceInfo() << IDevice::DeviceInfoItem(key, deviceStateToString());
 }
 
@@ -217,12 +308,12 @@ IDevice::DeviceInfo IDevice::deviceInformation() const
     \sa ProjectExplorer::IDeviceFactory
  */
 
-Core::Id IDevice::type() const
+Id IDevice::type() const
 {
     return d->type;
 }
 
-void IDevice::setType(Core::Id type)
+void IDevice::setType(Id type)
 {
     d->type = type;
 }
@@ -248,7 +339,7 @@ bool IDevice::isAutoDetected() const
     \sa ProjectExplorer::DeviceManager::findInactiveAutoDetectedDevice()
 */
 
-Core::Id IDevice::id() const
+Id IDevice::id() const
 {
     return d->id;
 }
@@ -263,6 +354,11 @@ bool IDevice::isCompatibleWith(const Kit *k) const
     return DeviceTypeKitAspect::deviceTypeId(k) == type();
 }
 
+QList<Task> IDevice::validate() const
+{
+    return {};
+}
+
 void IDevice::addDeviceAction(const DeviceAction &deviceAction)
 {
     d->deviceActions.append(deviceAction);
@@ -273,14 +369,9 @@ const QList<IDevice::DeviceAction> IDevice::deviceActions() const
     return d->deviceActions;
 }
 
-PortsGatheringMethod::Ptr IDevice::portsGatheringMethod() const
-{
-    return PortsGatheringMethod::Ptr();
-}
-
 DeviceProcessList *IDevice::createProcessListModel(QObject *parent) const
 {
-    Q_UNUSED(parent);
+    Q_UNUSED(parent)
     QTC_ASSERT(false, qDebug("This should not have been called..."); return nullptr);
     return nullptr;
 }
@@ -291,20 +382,14 @@ DeviceTester *IDevice::createDeviceTester() const
     return nullptr;
 }
 
-Utils::OsType IDevice::osType() const
+OsType IDevice::osType() const
 {
-    return Utils::OsTypeOther;
+    return d->osType;
 }
 
-DeviceProcess *IDevice::createProcess(QObject * /* parent */) const
+DeviceProcessSignalOperation::Ptr IDevice::signalOperation() const
 {
-    QTC_CHECK(false);
-    return nullptr;
-}
-
-DeviceEnvironmentFetcher::Ptr IDevice::environmentFetcher() const
-{
-    return DeviceEnvironmentFetcher::Ptr();
+    return {};
 }
 
 IDevice::DeviceState IDevice::deviceState() const
@@ -319,14 +404,14 @@ void IDevice::setDeviceState(const IDevice::DeviceState state)
     d->deviceState = state;
 }
 
-Core::Id IDevice::typeFromMap(const QVariantMap &map)
+Id IDevice::typeFromMap(const QVariantMap &map)
 {
-    return Core::Id::fromSetting(map.value(QLatin1String(TypeKey)));
+    return Id::fromSetting(map.value(QLatin1String(TypeKey)));
 }
 
-Core::Id IDevice::idFromMap(const QVariantMap &map)
+Id IDevice::idFromMap(const QVariantMap &map)
 {
-    return Core::Id::fromSetting(map.value(QLatin1String(IdKey)));
+    return Id::fromSetting(map.value(QLatin1String(IdKey)));
 }
 
 /*!
@@ -338,12 +423,13 @@ Core::Id IDevice::idFromMap(const QVariantMap &map)
 void IDevice::fromMap(const QVariantMap &map)
 {
     d->type = typeFromMap(map);
-    d->displayName = map.value(QLatin1String(DisplayNameKey)).toString();
-    d->id = Core::Id::fromSetting(map.value(QLatin1String(IdKey)));
+    d->displayName.fromMap(map, DisplayNameKey);
+    d->id = Id::fromSetting(map.value(QLatin1String(IdKey)));
     if (!d->id.isValid())
         d->id = newId();
     d->origin = static_cast<Origin>(map.value(QLatin1String(OriginKey), ManuallyAdded).toInt());
 
+    QWriteLocker locker(&d->lock);
     d->sshParameters.setHost(map.value(QLatin1String(HostKey)).toString());
     d->sshParameters.setPort(map.value(QLatin1String(SshPortKey), 22).toInt());
     d->sshParameters.setUserName(map.value(QLatin1String(UserNameKey)).toString());
@@ -351,25 +437,26 @@ void IDevice::fromMap(const QVariantMap &map)
     // Pre-4.9, the authentication enum used to have more values
     const int storedAuthType = map.value(QLatin1String(AuthKey), DefaultAuthType).toInt();
     const bool outdatedAuthType = storedAuthType
-            > QSsh::SshConnectionParameters::AuthenticationTypeSpecificKey;
+            > SshParameters::AuthenticationTypeSpecificKey;
     d->sshParameters.authenticationType = outdatedAuthType
-            ? QSsh::SshConnectionParameters::AuthenticationTypeAll
+            ? SshParameters::AuthenticationTypeAll
             : static_cast<AuthType>(storedAuthType);
 
-    d->sshParameters.privateKeyFile = map.value(QLatin1String(KeyFileKey), defaultPrivateKeyFilePath()).toString();
+    d->sshParameters.privateKeyFile =
+        FilePath::fromSettings(map.value(QLatin1String(KeyFileKey), defaultPrivateKeyFilePath()));
     d->sshParameters.timeout = map.value(QLatin1String(TimeoutKey), DefaultTimeout).toInt();
-    d->sshParameters.hostKeyCheckingMode = static_cast<QSsh::SshHostKeyCheckingMode>
-            (map.value(QLatin1String(HostKeyCheckingKey), QSsh::SshHostKeyCheckingNone).toInt());
+    d->sshParameters.hostKeyCheckingMode = static_cast<SshHostKeyCheckingMode>
+            (map.value(QLatin1String(HostKeyCheckingKey), SshHostKeyCheckingNone).toInt());
 
     QString portsSpec = map.value(PortsSpecKey).toString();
     if (portsSpec.isEmpty())
         portsSpec = "10000-10100";
-    d->freePorts = Utils::PortList::fromString(portsSpec);
+    d->freePorts = PortList::fromString(portsSpec);
     d->machineType = static_cast<MachineType>(map.value(QLatin1String(MachineTypeKey), DefaultMachineType).toInt());
     d->version = map.value(QLatin1String(VersionKey), 0).toInt();
 
-    d->debugServerPath = map.value(QLatin1String(DebugServerKey)).toString();
-    d->qmlsceneCommand = map.value(QLatin1String(QmlsceneKey)).toString();
+    d->debugServerPath = FilePath::fromSettings(map.value(QLatin1String(DebugServerKey)));
+    d->qmlRunCommand = FilePath::fromSettings(map.value(QLatin1String(QmlRuntimeKey)));
     d->extraData = map.value(ExtraDataKey).toMap();
 }
 
@@ -382,25 +469,26 @@ void IDevice::fromMap(const QVariantMap &map)
 QVariantMap IDevice::toMap() const
 {
     QVariantMap map;
-    map.insert(QLatin1String(DisplayNameKey), d->displayName);
+    d->displayName.toMap(map, DisplayNameKey);
     map.insert(QLatin1String(TypeKey), d->type.toString());
     map.insert(QLatin1String(IdKey), d->id.toSetting());
     map.insert(QLatin1String(OriginKey), d->origin);
 
+    QReadLocker locker(&d->lock);
     map.insert(QLatin1String(MachineTypeKey), d->machineType);
     map.insert(QLatin1String(HostKey), d->sshParameters.host());
     map.insert(QLatin1String(SshPortKey), d->sshParameters.port());
     map.insert(QLatin1String(UserNameKey), d->sshParameters.userName());
     map.insert(QLatin1String(AuthKey), d->sshParameters.authenticationType);
-    map.insert(QLatin1String(KeyFileKey), d->sshParameters.privateKeyFile);
+    map.insert(QLatin1String(KeyFileKey), d->sshParameters.privateKeyFile.toSettings());
     map.insert(QLatin1String(TimeoutKey), d->sshParameters.timeout);
     map.insert(QLatin1String(HostKeyCheckingKey), d->sshParameters.hostKeyCheckingMode);
 
     map.insert(QLatin1String(PortsSpecKey), d->freePorts.toString());
     map.insert(QLatin1String(VersionKey), d->version);
 
-    map.insert(QLatin1String(DebugServerKey), d->debugServerPath);
-    map.insert(QLatin1String(QmlsceneKey), d->qmlsceneCommand);
+    map.insert(QLatin1String(DebugServerKey), d->debugServerPath.toSettings());
+    map.insert(QLatin1String(QmlRuntimeKey), d->qmlRunCommand.toSettings());
     map.insert(ExtraDataKey, d->extraData);
 
     return map;
@@ -415,46 +503,52 @@ IDevice::Ptr IDevice::clone() const
     device->d->deviceState = d->deviceState;
     device->d->deviceActions = d->deviceActions;
     device->d->deviceIcons = d->deviceIcons;
+    // Os type is only set in the constructor, always to the same value.
+    // But make sure we notice if that changes in the future (which it shouldn't).
+    QTC_CHECK(device->d->osType == d->osType);
+    device->d->osType = d->osType;
     device->fromMap(toMap());
     return device;
 }
 
 QString IDevice::deviceStateToString() const
 {
-    const char context[] = "ProjectExplorer::IDevice";
     switch (d->deviceState) {
-    case IDevice::DeviceReadyToUse: return QCoreApplication::translate(context, "Ready to use");
-    case IDevice::DeviceConnected: return QCoreApplication::translate(context, "Connected");
-    case IDevice::DeviceDisconnected: return QCoreApplication::translate(context, "Disconnected");
-    case IDevice::DeviceStateUnknown: return QCoreApplication::translate(context, "Unknown");
-    default: return QCoreApplication::translate(context, "Invalid");
+    case IDevice::DeviceReadyToUse: return Tr::tr("Ready to use");
+    case IDevice::DeviceConnected: return Tr::tr("Connected");
+    case IDevice::DeviceDisconnected: return Tr::tr("Disconnected");
+    case IDevice::DeviceStateUnknown: return Tr::tr("Unknown");
+    default: return Tr::tr("Invalid");
     }
 }
 
-QSsh::SshConnectionParameters IDevice::sshParameters() const
+SshParameters IDevice::sshParameters() const
 {
+    QReadLocker locker(&d->lock);
     return d->sshParameters;
 }
 
-void IDevice::setSshParameters(const QSsh::SshConnectionParameters &sshParameters)
+void IDevice::setSshParameters(const SshParameters &sshParameters)
 {
+    QWriteLocker locker(&d->lock);
     d->sshParameters = sshParameters;
 }
 
 QUrl IDevice::toolControlChannel(const ControlChannelHint &) const
 {
     QUrl url;
-    url.setScheme(Utils::urlTcpScheme());
+    url.setScheme(urlTcpScheme());
+    QReadLocker locker(&d->lock);
     url.setHost(d->sshParameters.host());
     return url;
 }
 
-void IDevice::setFreePorts(const Utils::PortList &freePorts)
+void IDevice::setFreePorts(const PortList &freePorts)
 {
     d->freePorts = freePorts;
 }
 
-Utils::PortList IDevice::freePorts() const
+PortList IDevice::freePorts() const
 {
     return d->freePorts;
 }
@@ -469,32 +563,47 @@ void IDevice::setMachineType(MachineType machineType)
     d->machineType = machineType;
 }
 
-QString IDevice::debugServerPath() const
+FilePath IDevice::rootPath() const
+{
+    return FilePath::fromParts(u"device", id().toString(), u"/");
+}
+
+FilePath IDevice::debugServerPath() const
 {
     return d->debugServerPath;
 }
 
-void IDevice::setDebugServerPath(const QString &path)
+void IDevice::setDebugServerPath(const FilePath &path)
 {
     d->debugServerPath = path;
 }
 
-QString IDevice::qmlsceneCommand() const
+FilePath IDevice::debugDumperPath() const
 {
-    return d->qmlsceneCommand;
+    return d->debugDumperPath;
 }
 
-void IDevice::setQmlsceneCommand(const QString &path)
+void IDevice::setDebugDumperPath(const FilePath &path)
 {
-    d->qmlsceneCommand = path;
+    d->debugDumperPath = path;
 }
 
-void IDevice::setExtraData(Core::Id kind, const QVariant &data)
+FilePath IDevice::qmlRunCommand() const
+{
+    return d->qmlRunCommand;
+}
+
+void IDevice::setQmlRunCommand(const FilePath &path)
+{
+    d->qmlRunCommand = path;
+}
+
+void IDevice::setExtraData(Id kind, const QVariant &data)
 {
     d->extraData.insert(kind.toString(), data);
 }
 
-QVariant IDevice::extraData(Core::Id kind) const
+QVariant IDevice::extraData(Id kind) const
 {
     return d->extraData.value(kind.toString());
 }
@@ -515,13 +624,72 @@ QString IDevice::defaultPublicKeyFilePath()
     return defaultPrivateKeyFilePath() + QLatin1String(".pub");
 }
 
-void DeviceProcessSignalOperation::setDebuggerCommand(const QString &cmd)
+bool IDevice::ensureReachable(const FilePath &other) const
+{
+    return handlesFile(other); // Some first approximation.
+}
+
+expected_str<FilePath> IDevice::localSource(const Utils::FilePath &other) const
+{
+    Q_UNUSED(other);
+    return make_unexpected(Tr::tr("localSource() not implemented for this device type."));
+}
+
+bool IDevice::prepareForBuild(const Target *target)
+{
+    Q_UNUSED(target)
+    return true;
+}
+
+std::optional<Utils::FilePath> IDevice::clangdExecutable() const
+{
+    return std::nullopt;
+}
+
+void DeviceProcessSignalOperation::setDebuggerCommand(const FilePath &cmd)
 {
     m_debuggerCommand = cmd;
 }
 
 DeviceProcessSignalOperation::DeviceProcessSignalOperation() = default;
 
-DeviceEnvironmentFetcher::DeviceEnvironmentFetcher() = default;
+void DeviceProcessKiller::start()
+{
+    m_signalOperation.reset();
+    m_errorString.clear();
+
+    const IDevice::ConstPtr device = DeviceManager::deviceForPath(m_processPath);
+    if (!device) {
+        m_errorString = Tr::tr("No device for given path: \"%1\".").arg(m_processPath.toUserOutput());
+        emit done(false);
+        return;
+    }
+
+    m_signalOperation = device->signalOperation();
+    if (!m_signalOperation) {
+        m_errorString = Tr::tr("Device for path \"%1\" does not support killing processes.")
+                       .arg(m_processPath.toUserOutput());
+        emit done(false);
+        return;
+    }
+
+    connect(m_signalOperation.get(), &DeviceProcessSignalOperation::finished,
+            this, [this](const QString &errorMessage) {
+        m_errorString = errorMessage;
+        emit done(m_errorString.isEmpty());
+    });
+
+    m_signalOperation->killProcess(m_processPath.path());
+}
+
+KillerAdapter::KillerAdapter()
+{
+    connect(task(), &DeviceProcessKiller::done, this, &KillerAdapter::done);
+}
+
+void KillerAdapter::start()
+{
+    task()->start();
+}
 
 } // namespace ProjectExplorer
